
(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                          FUTSI ID MRSE
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)

(* Timer -- SML Basis Library

type cpu_timer
type real_timer

val startCPUTimer  : unit -> cpu_timer
val checkCPUTimer  : cpu_timer ->
                     { usr : Time.time, sys : Time.time, gc : Time.time }

val startRealTimer : unit -> real_timer
val checkRealTimer : real_timer -> Time.time

   [cpu_timer] is the type of timers for measuring CPU time consumption
   (user time, garbage collection time, and system time).

   [real_timer] is the type of timers for measuring the passing of
   real time (wall-clock time).

   [startCPUTimer ()] returns a cpu_timer started at the moment of
   the call.

   [checkCPUTimer tmr] returns {usr, sys, gc} where usr is the amount
   of user CPU time consumed since tmr was started, gc is the amount
   of user CPU time spent on garbage collection, and sys is the
   amount of system CPU time consumed since tmr was started.  Note
   that gc time is included in the usr time.  Under MS DOS, usr time
   and gc time are measured in real time.

   [startRealTimer ()] returns a real_timer started at the moment of
   the call.

   [checkRealTimer tmr] returns the amount of real time that has passed
   since tmr was started.
*)

(* Time -- SML Basis Library

eqtype time

exception Time

val fmt              : int -> time -> string

   [time] is a type for representing durations as well as absolute
   points in time (which can be thought of as durations since some
   fixed time zero).

   [zeroTime] represents the 0-second duration, and the origin of time,
   so zeroTime + t = t + zeroTime = t for all t.

   [fmt n t] returns as a string the number of seconds represented by
   t, rounded to n decimal digits.  If n <= 0, then no decimal digits
   are reported.

*)

(* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)

(* app load ["Timer", "Time"];
*)

(* idomero = fn : ('a -> 'b) -> 'a -> 'b * time * time
   idomero f a = az a argumentumra alkalmazott f fggvny eredmnybl,
                 tovbb a kirtkelshez felhasznlt CPU-idt s vals
                 idt msodpercben tartalmaz fzrekbl ll hrmas
*)
fun idomero f a =
    let
	val RealStart  = Timer.startRealTimer()
	val CPUstart   = Timer.startCPUTimer()
	val res = f a
	val {usr, ...} = Timer.checkCPUTimer CPUstart
	val rtm        = Timer.checkRealTimer RealStart
    in
	(res, usr, rtm)
    end;

(* idoiro = fn : string -> ('a -> 'b) -> 'a -> 'b
   idoiro t f a = f a
      mellkhatsknt kirja a t fzrt, az a argumentumra alkalmazott f
      fggvny kirtkelshez felhasznlt CPU-idt s vals idt (mp-ben)
*)
fun idoiro t f a = let val (res, usr, rtm) = idomero f a
		   in
		       (print (t ^ "   CPU time: " ^ Time.fmt 2 usr ^
			       ", real time: " ^ Time.fmt 2 rtm ^ " (sec)\n");
			res
			)
		   end;
