(* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                Szmtani sorozatok ellltsa
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ *)

(* Nav vltozat
   upto1 : int -> int -> int list
   upto1 m n = 1 klnbsg nvekv szmtani sorozat
               az alulrl zrt [m, n) tartomnyban
*)
fun upto1 m n = if m >= n then [] else m :: upto1 (m+1) n;

(* Nav vltozat, egy segdfggvnnyel
   upto2 : int -> int -> int list
   upto2 m n = 1 klnbsg nvekv szmtani sorozat
               az alulrl zrt [m, n) tartomnyban
*)
fun upto2 m n =
      let fun up m = if m >= n then [] else m :: up (m+1)
      in
	  up m
      end;

(* Nav vltozat d klnbsggel, kt segdfggvnnyel
   fmto1 : int -> int -> int -> int list
   fmto1 d m n = d>0 klnbsg, m<n esetn nvekv, m>n esetn cskken
     szmtani sorozat az alulrl, ill. fellrl zrt [m, n) tartomnyban
*)
fun fmto1 d m n =
      let fun up m = if n <= m then [] else m :: up (m+d)
	  fun dn m = if m <= n then [] else m :: dn (m-d)
      in
	  if      d <= 0      then []
	  else if n >= m      then up m
	  else (* if n < m then *) dn m
      end;

(* Nav vltozat d klnbsggel, egyetlen segdfggvnnyel
   fmto2 : int -> int -> int -> int list
   fmto2 d m n = d>0 klnbsg, m<n esetn nvekv, m>n esetn cskken
     szmtani sorozat az alulrl, ill. fellrl zrt [m, n) tartomnyban
*)
fun fmto2 d m n =
      let fun to cmp inc m = if cmp m then [] else m :: to cmp inc (inc m)
      in
	  if      d <= 0 then []
	  else if n >= m then      to (fn i => i >= n) (fn j => j + d) m
	  else (* if n < m then *) to (fn i => i <= n) (fn j => j - d) m
      end;

(* Akkumultoros vltozat d klnbsggel, egyetlen segdfggvnnyel
   fmto3 : int -> int -> int -> int list
   fmto3 d n m = d>0 klnbsg, m<n esetn nvekv, m>n esetn cskken
     szmtani sorozat az alulrl, ill. fellrl zrt [m, n) tartomnyban
*)
fun fmto3 d m n =
      let fun to cmp inc m zs = if cmp m then rev zs else to cmp inc (inc m) (m::zs)
      in
	  if      d <= 0 then []
	  else if n >= m then      to (fn i => i >= n) (fn j => j + d) m []
	  else (* if n < m then *) to (fn i => i <= n) (fn j => j - d) m []
      end;

(* Generikus akkumultoros vltozat komparl s klnbsgi fggvnnyel
   fmto4 : ('a * 'a -> bool) -> ('a -> 'a) -> 'a -> 'a -> 'a list
   fmto4 le df m n = a df klnbsgi fggvny s az le kisebb-vagy-egyenl
                     relci szerint nvekv sorozat az alulrl, ill.
                     fellrl zrt [m, n) tartomnyban
*)
fun fmto4 le df m n =
      let fun to cmp m zs = if cmp m then rev zs else to cmp (df m) (m::zs)
                 (* ha az eredmnylista sorrendje kzmbs, a rev elhagyhat *)
	  infix le
      in
	  if       m le df m andalso m le n (* df nvel s [m, n) nem res *)
	      then to (fn i => n le i) m []
	  else if  df m le m andalso n le m (* df cskkent s [m, n) nem res *)
	      then to (fn i => i le n) m []
	      else []
      end;

upto1 3 20;
upto1 ~3 20;

upto2 3 20;
upto2 3 ~20; (* res a sorozat *)
upto2 20 3;  (* res a sorozat *)

fmto1 2 3 20;
fmto1 2 ~20 3;
fmto1 2 3 ~20;
fmto1 2 20 3;
fmto1 ~2 20 3;

fmto2 2 3 20;
fmto2 2 ~20 3;
fmto2 2 3 ~20;
fmto2 2 20 3;
fmto2 ~2 20 3;

fmto3 2 3 20;
fmto3 2 ~20 3;
fmto3 2 3 ~20;
fmto3 2 20 3;
fmto3 ~2 20 3;

fmto4 op<= (fn i => i+2)  3  20;
fmto4 op<= (fn i => i+2) ~3  20;
fmto4 op<= (fn i => i-2) 20   3;
fmto4 op<= (fn i => i-2)  3  20;
fmto4 op<= (fn i => i-2) ~3  20;

fmto4 op<= (fn i => i+2.0)  3.4 20.1;
fmto4 op<= (fn i => i+2.0) ~3.4 20.1;
fmto4 op<= (fn i => i-2.0) 20.1  3.4;
fmto4 op<= (fn i => i-2.0)  3.4 20.1;
fmto4 op<= (fn i => i-2.0) ~3.4 20.1;

(* Akkumultoros vltozat, egy segdfggvnnyel
   upto : int -> int -> int list
   upto m n = 1 klnbsg nvekv szmtani sorozat
               az alulrl zrt [m, n) tartomnyban
*)
fun upto3 m n =
      let (* htulrl visszafel haladva ptjk a listt! *)
          fun up n zs = if m >= n then zs else up (n-1) (n-1::zs)
      in
          up n []
      end;

infix --;
fun fm -- to = upto3 fm to;

(* idmrs *)

val _ = idoiro "upto1, 100k int " upto1 1 100000;    (* "Out of memory" *)
val _ = idoiro "upto2, 100k int " upto2 1 100000;    (* "Out of memory" *)
val _ = idoiro "fmto1, 100k int " fmto1 1 1 100000;  (* "Out of memory" *)
val _ = idoiro "fmto2, 100k int " fmto2 1 1 100000;  (* "Out of memory" *)

val _ = idoiro "upto3, 1M int " upto3 1 1000000;                          (* Akkumulatoros *)
val _ = idoiro "fmto3, 1M int " fmto3 1 1 1000000;                        (* Akkumulatoros *)
val _ = idoiro "fmto4, 1M int " fmto4 op<= (fn i => i+1)   1   1000000;   (* Akkumulatoros *)
val _ = idoiro "fmto4, 1M real" fmto4 op<= (fn i => i+1.0) 1.0 1000000.0; (* Akkumulatoros *)

(* polyML

PolyML.timing(true);

val _ = upto1 1 1000000;    (* "Increasing stack" *)
val _ = upto2 1 1000000;    (* "Increasing stack" *)
val _ = fmto1 1 1 1000000;  (* "Increasing stack" *)
val _ = fmto2 1 1 1000000;  (* "Increasing stack" *)

val _ = upto3 1 1000000;                          (* Akkumulatoros *)
val _ = fmto3 1 1 1000000;                        (* Akkumulatoros *)
val _ = fmto4 op<= (fn i => i+1)   1   1000000;   (* Akkumulatoros *)
val _ = fmto4 op<= (fn i => i+1.0) 1.0 1000000.0; (* Akkumulatoros *)

PolyML.timing(false);

*)
